@@ -136,12 +136,11 @@ const char * const server_names[] = {
 #ifdef XORG
     "Xorg        Common X server for most displays",
 #endif
-#ifdef __DARWIN__
-    "XDarwin         Darwin/Mac OS X IOKit displays",
-    "XDarwinQuartz   Mac OS X Quartz displays",
-    "XDarwinStartup  Auto-select between XDarwin and XDarwinQuartz",
+#ifdef __APPLE__
+    "Xquartz     Mac OSX Quartz displays.",
 #endif
     "Xvfb        Virtual frame buffer",
+    "Xfake       kdrive-based virtual frame buffer",
     "Xnest       X server nested in a window on another X server",
     "Xephyr      kdrive-based nested X server",
     NULL};
@@ -161,18 +160,18 @@ char xserverrcbuf[256];
 #define	OK_EXIT		0
 #define	ERR_EXIT	1
 
-char *default_server = "X";
-char *default_display = ":0";		/* choose most efficient */
-char *default_client[] = {"xterm", "-geometry", "+1+1", "-n", "login", NULL};
-char *serverargv[100];
-char *clientargv[100];
-char **server = serverargv + 2;		/* make sure room for sh .xserverrc args */
-char **client = clientargv + 2;		/* make sure room for sh .xinitrc args */
-char *displayNum;
-char *program;
-Display *xd;			/* server connection */
+static char *default_server = "X";
+static char *default_display = ":0";		/* choose most efficient */
+static char *default_client[] = {"xterm", "-geometry", "+1+1", "-n", "login", NULL};
+static char *serverargv[100];
+static char *clientargv[100];
+static char **server = serverargv + 2;		/* make sure room for sh .xserverrc args */
+static char **client = clientargv + 2;		/* make sure room for sh .xinitrc args */
+static char *displayNum = NULL;
+static char *program = NULL;
+static Display *xd = NULL;			/* server connection */
 #ifndef SYSV
-#if defined(__CYGWIN__) || defined(SVR4) || defined(_POSIX_SOURCE) || defined(CSRG_BASED) || defined(__UNIXOS2__) || defined(Lynx)
+#if defined(__CYGWIN__) || defined(SVR4) || defined(_POSIX_SOURCE) || defined(CSRG_BASED) || defined(__UNIXOS2__) || defined(Lynx) || defined(__APPLE__)
 int status;
 #else
 union wait	status;
@@ -207,7 +206,7 @@ sigCatch(int sig)
 static SIGVAL 
 sigAlarm(int sig)
 {
-#if defined(SYSV) || defined(SVR4) || defined(linux) || defined(__UNIXOS2__)
+#if defined(SYSV) || defined(SVR4) || defined(linux) || defined(__UNIXOS2__) || defined(__APPLE__)
 	signal (sig, sigAlarm);
 #endif
 }
@@ -215,7 +214,7 @@ sigAlarm(int sig)
 static SIGVAL
 sigUsr1(int sig)
 {
-#if defined(SYSV) || defined(SVR4) || defined(linux) || defined(__UNIXOS2__)
+#if defined(SYSV) || defined(SVR4) || defined(linux) || defined(__UNIXOS2__) || defined(__APPLE__)
 	signal (sig, sigUsr1);
 #endif
 }
@@ -355,10 +354,11 @@ main(int argc, char *argv[], char *envp[])
 
 	    xinitrcbuf[0] = '\0';
 	    if ((cp = getenv ("XINITRC")) != NULL) {
-		strcpy (xinitrcbuf, cp);
+		(void) snprintf (xinitrcbuf, sizeof(xinitrcbuf), "%s", cp);
 		required = True;
 	    } else if ((cp = getenv ("HOME")) != NULL) {
-		(void) sprintf (xinitrcbuf, "%s/%s", cp, XINITRC);
+		(void) snprintf (xinitrcbuf, sizeof(xinitrcbuf),
+				 "%s/%s", cp, XINITRC);
 	    }
 	    if (xinitrcbuf[0]) {
 		if (access (xinitrcbuf, F_OK) == 0) {
@@ -382,10 +382,11 @@ main(int argc, char *argv[], char *envp[])
 
 	    xserverrcbuf[0] = '\0';
 	    if ((cp = getenv ("XSERVERRC")) != NULL) {
-		strcpy (xserverrcbuf, cp);
+		(void) snprintf (xserverrcbuf, sizeof(xserverrcbuf), "%s", cp);
 		required = True;
 	    } else if ((cp = getenv ("HOME")) != NULL) {
-		(void) sprintf (xserverrcbuf, "%s/%s", cp, XSERVERRC);
+		(void) snprintf (xserverrcbuf, sizeof(xserverrcbuf),
+				 "%s/%s", cp, XSERVERRC);
 	    }
 	    if (xserverrcbuf[0]) {
 		if (access (xserverrcbuf, F_OK) == 0) {
@@ -399,7 +400,6 @@ main(int argc, char *argv[], char *envp[])
 	    }
 	}
 
-
 	/*
 	 * put the display name into the environment
 	 */
@@ -496,7 +496,7 @@ processTimeout(int timeout, char *string)
 			break;
 		alarm(0);
 #else /* SYSV */
-#if defined(SVR4) || defined(_POSIX_SOURCE) || defined(Lynx)
+#if defined(SVR4) || defined(_POSIX_SOURCE) || defined(Lynx) || defined(__APPLE__)
 		if ((pidfound = waitpid(serverpid, &status, WNOHANG)) == serverpid)
 			break;
 #else
@@ -649,6 +649,7 @@ setWindowPath(void)
 	unsigned long num;
 	char nums[10];
 	int numn;
+	size_t len;
 	prop = XInternAtom(xd, "XFree86_VT", False);
 	if (prop == None) {
 #ifdef DEBUG
@@ -704,11 +705,18 @@ setWindowPath(void)
 	windowpath = getenv("WINDOWPATH");
 	numn = snprintf(nums, sizeof(nums), "%lu", num);
 	if (!windowpath) {
-		newwindowpath = malloc(10 + 1 + numn + 1);
-		sprintf(newwindowpath, "WINDOWPATH=%s", nums);
+		len = 10 + 1 + numn + 1;
+		newwindowpath = malloc(len);
+		if (newwindowpath == NULL) 
+		    return;
+		snprintf(newwindowpath, len, "WINDOWPATH=%s", nums);
 	} else {
-		newwindowpath = malloc(10 + 1 + strlen(windowpath) + 1 + numn + 1);
-		sprintf(newwindowpath, "WINDOWPATH=%s:%s", windowpath, nums);
+		len = 10 + 1 + strlen(windowpath) + 1 + numn + 1;
+		newwindowpath = malloc(len);
+		if (newwindowpath == NULL)
+		    return;
+		snprintf(newwindowpath, len, "WINDOWPATH=%s:%s", 
+			 windowpath, nums);
 	}
 	*newenvironlast++ = newwindowpath;
 	*newenvironlast = NULL;
@@ -816,7 +824,7 @@ set_environment(void)
 {
     int nenvvars;
     char **newPtr, **oldPtr;
-    static char displaybuf[256];
+    static char displaybuf[512];
 
     /* count number of environment variables */
     for (oldPtr = environ; *oldPtr; oldPtr++) ;
@@ -831,8 +839,7 @@ set_environment(void)
     }
 
     /* put DISPLAY=displayname as first element */
-    strcpy (displaybuf, "DISPLAY=");
-    strcpy (displaybuf + 8, displayNum);
+    snprintf (displaybuf, sizeof(displaybuf), "DISPLAY=%s", displayNum);
     newPtr = newenviron;
     *newPtr++ = displaybuf;
 
